﻿--[[
	Persistent Data for RaidTracker, Saved Variables
	
	Aribrary definitions to keep things clear on identifiers:
		Group   - the set of 5 or less members in a party, raid or BG, or any set of members in any type of group
		Party   - group of 5 or less that is not in a raid and is using the Party UI
		Raid    - a set of one or more groups that has formed into a raid and is using the Raid UI
		BG      - a raid that is a pvp raid or party and is using the PvP UI (BattleGroup)
		
		Rarity - the quality or rarity of an item, quality and rarity are used interchangably
		         rarity is chosen for identifier because using word quality by self could be ambiguous
			[RarityID]  1:poor, 2:common, 3:uncommon, 4:rare, 5:epic, 6:legendary, 7:artifact
]]


KARaidTracker = KarmaLibrary:GetInstance( KARaidTracker, "RaidTracker" );

KARaidTracker:RegisterDB( "KARaidTrackerDB", {
	Options = {
		AutoRaid = 2,                   -- [0:off|1:log|2:create]  logging raid if a raid
		AutoParty = 1,					-- [0:off|1:log|2:create]  logging raid if a party
		AutoSolo = 1,					-- [0:off|1:log|2:create]  logging raid if solo
		AutoBattlegroup = 0,			-- [0:off|1:log|2:create]  logging raid if a battelgroup
		AutoArena = 0,					-- [0:off|1:log|2:create]  logging raid if a battelgroup
		AutoBoss = 1,                   -- [0:off|1:onMouseover|2:onBossKill]  when to check for auto boss
		AutoZone = 1,                   -- [0|1]  automatically determine zone
		BossAsk = 0,                    -- ask on boss kill whats the next boss is
		BossName = "Trash mob",         -- just the name of the boss
		BossCooldown = 120,			    -- [seconds]  how long should trash mob's ignored after a boss kill
		CostAsk = 0,                    -- [RarityID|0:off]  minimum item rarity to ask for dkp cost for items (ask user)
		CostGet = 0,                	-- [RarityID|0:off]  minimum item rarity to get for dkp cost for items (get from plugin)
		ExportFormat = 2,               -- [0:MLdkp1.1/EQdkpPlugin|1:EQdkpUnknown|2:MLdkp1.5]  Export format for dkp
		GroupItems = 3,                 -- [RarityID|0:off]  maximum item rarity to group items by count instead of log individualy
		LogAttendees = 1,               -- [0:off|1:onLoot|2:onBoss]  log attendees in database on specified event
		LogGuild = 1,                   -- [0|1]  Snapshots the guildroster on bosskill
		MaxLevel = 70,                  -- If player lvl is maxlevel it will not be exported to mldkp
		LogItems = 3,                   -- [RarityID|0:off]
		SavePlayerDetail = 1,			-- on/off - save race, class and level  
		SaveTooltips = 1,               -- on/off - save tooltips of items
		TimeFormat = 1,                 -- on/off - Use 24h time format
		Timezone = 0,					-- logging players timezone offset
		TimeSync = 1,
		WipeAsk = 0,                    -- ask if the group dies if it is a wipe, if all are dead it will not ask
		WipePercent = 0.5,              -- how many prozent of group must be dead to ask
		WipeCooldown = 150,             -- how long should death be ignored after a wipe count (seconds)
		-- Auditing
		DebugMode = 0,                  -- [0|1]  Debug mode for development
		-- Internal flags
		VersionFix = 11,                -- [0...]  Current database version level
		VersionCopy = 0,                -- [0|1]  Is imported CT_RaidTracker, one time thing very first load only
		TimeOffsetStatus = 0,           -- ???
		ItemOptionsSelected = nil,      -- ???
		CurrentRaid = nil,				-- raid id of raid currently beign logged
	},	
    SortOptions = {
        method = "name",
        way = "asc",
        itemmethod = "looted",
        itemway = "asc",
        itemsfilter = 1,
        playerraidway = "desc",
        playeritemsfilter = 1,
        playeritemmethod = "name",
        playeritemway = "asc",
        itemhistorymethod = "name",
        itemhistoryway = "asc",
    },
    ItemOptions = { 
		-- BC
		[29434] = { id = 29434, name = "Badge of Justice",        CostGet = 0, Track = 0, Group = 0, CostAsk = 0, quality = 4, },
		--   TK - Kael'thas
		[30312] = { id = 30311, name = "Warp Slicer",             CostGet = 0, Track = 0, Group = 0, CostAsk = 0, quality = 5, },
		[30312] = { id = 30312, name = "Infinity Blade",          CostGet = 0, Track = 0, Group = 0, CostAsk = 0, quality = 5, },
		[30313] = { id = 30313, name = "Staff of Disintegration", CostGet = 0, Track = 0, Group = 0, CostAsk = 0, quality = 5, },
		[30314] = { id = 30314, name = "Phaseshift Bulwark",      CostGet = 0, Track = 0, Group = 0, CostAsk = 0, quality = 5, },
		[30316] = { id = 30316, name = "Devastation",			  CostGet = 0, Track = 0, Group = 0, CostAsk = 0, quality = 5, },
		[30317] = { id = 30317, name = "Cosmic Infuser",          CostGet = 0, Track = 0, Group = 0, CostAsk = 0, quality = 5, },
		[30318] = { id = 30318, name = "Netherstrand Longbow",    CostGet = 0, Track = 0, Group = 0, CostAsk = 0, quality = 5, },
		[30319] = { id = 30319, name = "Nether Spike",            CostGet = 0, Track = 0, Group = 0, CostAsk = 0, quality = 5, },
		--   BT - Naj'entus
		[32408] = { id = 32408, name = "Naj'entus Spine",         CostGet = 0, Track = 0, Group = 0, CostAsk = 0, quality = 2, },
	},
    ZoneTriggers = RaidTracker_ZoneTriggers,
    Online = { },
    RaidLog = { },
} );


function KARaidTracker:RunVersionFix( frame )
	local db = self._store.db;

    if db.Options.VersionCopy == 0 then
    	self:Print("RaidTracker: Importing CT database start.")
    	self:ImportToDB( "Options", _G, "CT_RaidTracker_", false, { 
			"VersionFix",CurrentRaid="GetCurrentRaid","MinQuality","AutoRaidCreation","GroupItems",DkpValue="GetDkpValue","OldFormat",
			"AutoBoss","AutoBossBoss","AutoZone",CostAsk="AskCosts","Timezone","TimeSync",DebugMode="DebugFlag" } );    	        
    	self:ImportToDB( nil, _G, "CT_RaidTracker_", true, {"Options","ItemOptions","SortOptions"} );
    	self:ImportToDB( nil, _G, "CT_RaidTracker_", false, {"RaidLog","CustomZoneTriggers","Online"} );
    	self:Print("RaidTracker: Importing CT database end.")
	    db.Options.VersionCopy = 1;
    end
	    
	local opts = db.Options;
    local log = db.RaidLog;
    
    if opts.VersionFix == 0 then
        self:Debug("VersionFix", 1);
        for k, v in pairs(log) do
            if(not log[k].PlayerInfos) then
                log[k].PlayerInfos = { };
            end
            if ( v["Notes"] ) then
                for notesk, notesv in pairs(v["Notes"]) do
                    if(not log[k].PlayerInfos[notesk]) then
                        log[k].PlayerInfos[notesk] = { };
                    end
                    log[k].PlayerInfos[notesk]["note"] = notesv;
                    self:Debug("VersionFix", 1, "note", k, notesk, notesv);
                end
                log[k]["Notes"] = nil;
            end
        end
        opts.VersionFix = 1;
    end
    if opts.VersionFix == 1 then
        self:Debug("VersionFix", 2);
        opts.VersionFix = 2; -- Do not remove tooltips any longer
    end
    if opts.VersionFix == 2 then
        self:Debug("VersionFix", 3);        
        for k, v in pairs(log) do
            if(log[k].BossKills and getn(log[k].BossKills) >= 1 and not log[k].BossKills[1]) then
                local tempbosskills = {};
                for k2, v2 in pairs(log[k].BossKills) do
                    tempbosskills[k2] = v2;
                end
                log[k].BossKills = {};
                for k2, v2 in pairs(tempbosskills) do
                    tinsert(log[k].BossKills,
                        {
                            ["boss"] = k2,
                            ["time"] = v2,
                            ["attendees"] = {}
                        }
                    );
                    self:Debug("VersionFix", 3, "BossKills", k, k2);
                end
            end
        end
        opts.VersionFix = 3;
    end
    if opts.VersionFix == 3 then
        self:Debug("VersionFix", 4);
        for k, v in pairs(log) do
            if(log[k].BossKills) then
                for k2, v2 in pairs(log[k].BossKills) do
                    if(type(v2.time) == "table") then
                        self:Debug("VersionFix", 4, "BossKills Fix", k, k2, v2.time.boss, v2.time.time);
                        log[k].BossKills[k2].boss = v2.time.boss;
                        log[k].BossKills[k2].attendees = v2.time.attendees;
                        log[k].BossKills[k2].time = v2.time.time;
                    end
                end
            end
        end
        opts.VersionFix = 4;
    end
    if opts.VersionFix == 4 then
        self:Debug("VersionFix", 5);
        opts.Wipe = 1;                     -- ask if the group dies if it is a wipe, if all are dead it will not ask
        opts.WipePercent = 0.5;            -- how many prozent of group must be dead to ask
        opts.NextBoss = 1;                 -- ask on boss kill whats the next boss is
        
        opts.VersionFix = 5;
    end
    if opts.VersionFix == 5 then
        self:Debug("VersionFix", 6);
        opts.MLdkp = 1;                     -- Export mldkp
        
        opts.VersionFix = 6;
    end;
    if opts.VersionFix == 6 then
        self:Debug("VersionFix", 7);
        if (opts.MLdkp and opts.MLdkp == 1) then
            opts.ExportFormat = 2;
        else
            if (opts.OldFormat and opts.OldFormat == 1) then
                opts.ExportFormat = 0;
            else
                opts.ExportFormat = 1;
            end;
        end;
        opts.MLdkp = nil;
        opts.OldFormat = nil;

        opts.VersionFix = 7;
    end;
-- Post CT RaidTracker updates 
    if opts.VersionFix == 7 then    	
        self:Debug("VersionFix", 8);
        self:ImportRenamesDB( opts, { TimeFormat="24hFormat", CurrentRaid="GetCurrentRaid", DkpValue="GetDkpValue", } );
		
        opts.VersionFix = 8;
    end;
    if opts.VersionFix == 8 or opts.VersionFix == 9 then
        self:Debug("VersionFix", 9, 10);
        self:ImportRenamesDB( opts, { itemsfilter="itemfilter", playeritemsfilter="playeritemfilter", DebugMode="DebugFlag", } );
		if not opts.DebugMode then opts.DebugMode = 0; end

        opts.VersionFix = 10;
    end;
    if opts.VersionFix == 10 then
        self:Debug("VersionFix", 11);
        -- options
        self:ImportRenamesDB( opts, { SavePlayerDetail="SaveExtendedPlayerInfo", CostAsk="AskCosts", CostGet="DkpValue", WipeAsk="Wipe", 
			WipeCooldown="WipeCoolDown", AutoParty="LogGroup", AutoRaid="AutoRaidCreation", BossName="AutoBossBoss",
			BossCooldown="AutoBossChangeMinTime", LogGuild="GuildSnapshot", BossAsk="NextBoss", LogItems="MinQuality",
			AutoBattlegroup="LogBattlefield", } );
		if opts.AutoGroup == 1 then opts.AutoParty = 2 end; opts.AutoGroup = nil;
		if opts.AutoRaid == 1 then opts.AutoRaid = 2 end;
		if opts.AutoBattelgroup == 1 then opts.AutoArena = 1 end;
		-- item options
		local t = { Track="status", CostAsk="askcosts", CostGet="costgrabbing", Group="group", }
		for key, val in pairs(db.ItemOptions) do
			if type(val)=="table" then
				self:ImportRenamesDB( val, t );		
				table.remove(db.ItemOptions, key);
				db.ItemOptions[val.id] = val;
			end
		end
		-- zone triggers
    	self:ImportToDB( nil, nil, nil, false, {ZoneTriggers="CustomZoneTriggers"} );
		-- raid log
		for k, v in pairs(log) do
			for k1, v1 in pairs(v.Loot) do
				if v1.player == "disenchanted" then v1.player = "disenchant" end
				v1.id = k1;
				if v1.thisitemid then v1.thisitemid = nil end
			end
		end

        opts.VersionFix = 11;
    end;
    
end


